# Malaya Natural Language Toolkit
#
# Copyright (C) 2019 Malaya Project
# Licensed under the MIT License
# Author: huseinzol05 <husein.zol05@gmail.com>
# URL: <https://malaya.readthedocs.io/>
# For license information, see https://github.com/huseinzol05/Malaya/blob/master/LICENSE

from ._tatabahasa import (
    tanya_list,
    perintah_list,
    pangkal_list,
    bantu_list,
    penguat_list,
    penegas_list,
    nafi_list,
    pemeri_list,
    sendi_list,
    pembenar_list,
    nombor_list,
    suku_bilangan_list,
    pisahan_list,
    keterangan_list,
    arah_list,
    hubung_list,
    gantinama_list,
    permulaan,
    hujung,
    hujung_malaysian,
    calon_dictionary,
    stopwords_calon,
)
from ._text_functions import STOPWORDS as stopwords
from ._text_functions import split_into_sentences as sentence_splitter
