english = {
    'afaik': 'as far as i know',
    'afk': 'away from keyboard',
    'asap': 'as soon as possible',
    'atk': 'at the keyboard',
    'atm': 'at the moment',
    'a3': 'anytime, anywhere, anyplace',
    'bak': 'back at keyboard',
    'bbl': 'be back later',
    'bbs': 'be back soon',
    'bfn': 'bye for now',
    'b4n': 'bye for now',
    'brb': 'be right back',
    'brt': 'be right there',
    'btw': 'by the way',
    'b4': 'before',
    'cu': 'see you',
    'cul8r': 'see you later',
    'cya': 'see you',
    'faq': 'frequently asked questions',
    'fc': 'fingers crossed',
    'fwiw': 'for what its worth',
    'fyi': 'for your information',
    'gal': 'get a life',
    'gg': 'good game',
    'gn': 'good night',
    'gmta': 'great minds think alike',
    'gr8': 'great!',
    'g9': 'genius',
    'ic': 'i see',
    'icq': 'i seek you',
    'ilu': 'ilu: i love you',
    'imho': 'in my humble opinion',
    'imo': 'in my opinion',
    'iow': 'in other words',
    'irl': 'in real life',
    'kiss': 'keep it simple, stupid',
    'ldr': 'long distance relationship',
    'lmao': 'laugh my ass off',
    'lol': 'laughing out loud',
    'ltns': 'long time no see',
    'l8r': 'later',
    'mte': 'my thoughts exactly',
    'm8': 'mate',
    'nrn': 'no reply necessary',
    'oic': 'oh i see',
    'pita': 'pain in the ass',
    'prt': 'party',
    'prw': 'parents are watching',
    'qpsa': 'que pasa',
    'rofl': 'rolling on the floor laughing',
    'roflol': 'rolling on the floor laughing out loud',
    'rotflmao': 'rolling on the floor laughing my ass off',
    'sk8': 'skate',
    'stats': 'your sex and age',
    'asl': 'age sex location',
    'thx': 'thank you',
    'ty': 'thank you',
    'ttfn': 'ta-ta for now!',
    'ttyl': 'talk to you later',
    'u': 'you',
    'u2': 'you too',
    'u4e': 'yours for ever',
    'wb': 'welcome back',
    'wtf': 'what the fuck',
    'wtg': 'way to go!',
    'wuf': 'where are you from?',
    'w8': 'wait',
    '7k': 'sick laugher',
    'r': 'are',
    'w': 'wait',
    'hrs': 'hours',
    'd': 'the',
    'fwd': 'forward',
    'cant': 'cannot',
    'hve': 'have',
    'cause': 'because',
    'couldve': 'could have',
    've': 'have',
    'nt': 'not',
    'gonna': 'going to',
    'id': 'i would',
    'maam': 'madam',
    'nothin': 'nothing',
    'fu': 'fuck you',
    'aamof': 'as a matter of fact',
    'ack': 'acknowledgment',
    'ot': 'off topic',
    'op': 'original poster original post',
    'otoh': 'on the other hand',
    'pov': 'point of view',
    'rotfl': 'rolling on the floor laughing',
    'rtfm': 'read the fine manual',
    'fck': 'fuck',
    'tyvm': 'thank you very much',
    'tyt': 'take your time',
    'yeo fyeo': 'for your eyes only',
    'afair': 'as far as i remember recall',
    'aka': 'also known as',
    'bkbtk': 'back to keyboard',
    'btt': 'back to topic',
    'bc': 'because',
    'cys': 'check your settings',
    'diy': 'do it yourself',
    'eobd': 'end of business day',
    'eod': 'end of day',
    'eom': 'end of message',
    'eot': 'end of thread text transmission',
    'fack': 'full acknowledge',
    'fka': 'formerly known as',
    'fyi jfyi': 'justfor your information',
    'ftw': 'fuck the world for the win',
    'hf': 'have fun',
    'hth': 'hope this helps',
    'idk': 'i dont know',
    'iirc': 'if i recall remember correctly',
    'imnsho': 'in my not so humble honest opinion',
    'itt': 'in this thread',
    'dgmw': 'dont get me wrong',
    'mmw': 'mark my words',
    'na': 'not available applicable',
    'nan': 'not a number',
    'nntr': 'no need to reply',
    'noyb': 'none of your business',
    'omg': 'oh my god',
    'pebkac': 'problem exists between keyboard and chair',
    'rsvp repondez sil vous plait french': 'please reply',
    'scnr': 'sorry could not resist',
    'sflr': 'sorry for late reply',
    'spoc': 'single point of contact',
    'tba': 'to be announced',
    'tbc': 'to be continued to be confirmed',
    'tia': 'thanks in advance',
    'tgif': 'thanks god its friday',
    'thx tnx': 'thanks',
    'tq': 'thank you',
    'wt': 'whoomp there it is',
    'wfm': 'works for me',
    'wrt': 'with regard to',
    'wth': 'what the hell what the heck',
    'ymmd': 'you made my day',
    'ymmv': 'your mileage may vary',
    'yam': 'yet another meeting',
    'icymi': 'in case you missed it',
    'moro': 'tomorrow',
    'nte': 'tonight',
    'aeap': 'as early as possible',
    'alap': 'as late as possible',
    'b': 'blah blah blah',
    'byki': 'before you know it',
    'bff': 'best friends forever',
    'btam': 'be that as it may',
    'ctn': 'cannot talk now',
    'cus': 'see you soon',
    'cwot': 'complete waste of time',
    'cyt': 'see you tomorrow',
    'em': 'excuse me',
    'ff': 'face to face',
    'foaf': 'friend of a friend',
    'gr': 'great',
    'hak': 'hugs and kisses',
    'idc': 'i dont care',
    'ilu ily': 'i love you',
    'imu': 'i miss you',
    'jk': 'just kidding',
    'jc': 'just checking',
    'jtlyk': 'just to let you know',
    'kfy': 'kiss for you',
    'kmn': 'kill me now',
    'kpc': 'keeping parents clueless',
    'lr': 'later',
    'mof': 'male or female',
    'mtfbwy': 'may the force be with you',
    'myob': 'mind your own business',
    'naz': 'name address zip',
    'nc': 'no comment',
    'nimby': 'not in my backyard',
    'nm': 'never mind nothing much',
    'np': 'no problem',
    'nsfw': 'not safe for work',
    'ntim': 'not that it matters',
    'nvm': 'never mind',
    'oatus': 'on a totally unrelated subject',
    'omw': 'on my way',
    'otl': 'out to lunch',
    'otp': 'on the phone',
    'p': 'parent alert',
    'pal': 'parents are listening',
    'paw': 'parents are watching',
    'pir': 'parent in room',
    'pos': 'parent over shoulder',
    'props': 'proper respect proper recognition',
    'qt': 'cutie',
    'rn': 'right now',
    'ru': 'are you',
    'sep': 'someone elses problem',
    'sitd': 'still in the dark',
    'slap': 'sounds like a plan',
    'smim': 'send me an instant message',
    'so': 'significant other',
    'tmi': 'too much information',
    'ur': 'your you are',
    'wycm': 'will you call me',
    'wywh': 'wish you were here',
    'xoxoxox': 'hugs kisses',
}
