pip3 install setuptools -U
pip3 install tqdm unidecode numpy scipy python-levenshtein tensorflow pyldavis
pip3 install sklearn_crfsuite sklearn scikit-learn requests fuzzywuzzy
pip3 install pandas toolz
pip3 install PySastrawi matplotlib seaborn networkx
python3 setup.py install
